@@ -1615,6 +1615,7 @@ class BIP32_Wallet(Deterministic_Wallet):
     def mnemonic_to_seed(self, seed, password):
         return Mnemonic.mnemonic_to_seed(seed, password)
 
+    @classmethod
     def make_seed(self, lang=None):
         return Mnemonic(lang).make_seed()
 
@@ -1986,6 +1987,10 @@ class Wallet(object):
     def is_seed(seed):
         return is_old_seed(seed) or is_new_seed(seed)
 
+    @staticmethod
+    def is_mpk(text):
+        return Wallet.is_old_mpk(text) or Wallet.is_xpub(text)
+
     @staticmethod
     def is_old_mpk(mpk):
         try:
